package com.sd.business.controller.inventory;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.annotation.NonInterception;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByDepartmentDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.dto.QuantityDto;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.service.inventory.InventoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 库存 前端控制器
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@RestController
@RequestMapping("/inventory")
public class InventoryController {

    @Autowired
    private InventoryService inventoryService;

    /**
     * 库存分页
     */
    @PostMapping("/page")
    public Page<InventoryVo> page(@RequestBody InventorySelectDto dto) {
        return inventoryService.getPage(dto);
    }

    /**
     * 库存导出
     */
    @NonInterception
    @PostMapping("/exportExcel")
    public void exportExcel(@RequestBody InventorySelectDto dto) {
        inventoryService.exportExcel(dto);
    }

    /**
     * 查询库存
     */
    @PostMapping("/getQuantity")
    public Map<Long, BigDecimal> getQuantity(@Validated @RequestBody QuantityDto dto) {
        return inventoryService.getQuantity(dto);
    }

    /**
     * 根据仓库获取库存数量
     */
    @PostMapping("/getQuantityByWarehouse")
    public List<QuantityByWarehouseVo> getQuantityByWarehouse(@RequestBody QuantityByWarehouseDto dto) {
        return inventoryService.getQuantityByWarehouse(dto);
    }

    /**
     * 根据事业部获取库存数量
     */
    @PostMapping("/getQuantityByDepartment")
    public List<QuantityByDepartmentVo> getQuantityByDepartment(@RequestBody QuantityByDepartmentDto dto) {
        return inventoryService.getQuantityByDepartment(dto);
    }

}
