package com.sd.business.entity.in.vo;

import com.alibaba.excel.annotation.ExcelIgnoreUnannotated;
import com.alibaba.excel.annotation.ExcelProperty;
import com.alibaba.excel.annotation.write.style.ColumnWidth;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.OutDetailTypeEnum;
import lombok.Getter;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Objects;

/**
 * 出入库流水导出实体
 *
 * @author
 * @since 2023-10-08
 */
@Getter
@Setter
@ExcelIgnoreUnannotated
public class InOutStorageBomExportVo {
    /**
     * 出入库单号
     */
    @ColumnWidth(20)
    @ExcelProperty(value = "出入库单号", index = 0)
    private String code;

    /**
     * 事业部名称
     */
    @ColumnWidth(20)
    @ExcelProperty(value = "归属事业部", index = 1)
    private String departmentName;

    /**
     * 仓库名称
     */
    @ColumnWidth(20)
    @ExcelProperty(value = "仓库", index = 2)
    private String warehouseName;

    /**
     * 类型 1入库 0出库
     */
    private Integer type;

    /**
     * 出入库明细
     */
    private Integer detailType;

    /**
     * 出入库明细类型值
     */
    @ColumnWidth(20)
    @ExcelProperty(value = "出入库类型", index = 3)
    private String detailTypeValue;

    /**
     * 产品规格品号
     */
    @ColumnWidth(20)
    @ExcelProperty(value = "品号", index = 4)
    private String bomSpecCode;

    /**
     * 产品规格品名
     */
    @ColumnWidth(20)
    @ExcelProperty(value = "品名", index = 5)
    private String bomSpecName;

    /**
     * 数量
     */
    @ExcelProperty(value = "数量", index = 6)
    private BigDecimal quantity;

    /**
     * 结存库存数量
     */
    @ColumnWidth(15)
    @ExcelProperty(value = "库存数量", index = 7)
    private BigDecimal balanceInventoryQuantity;

    /**
     * 出入库时间
     */
    @ColumnWidth(20)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @ExcelProperty(value = "操作时间", index = 8)
    private Date inOutStorageTime;

    /**
     * 申请人
     */
    @ColumnWidth(20)
    @ExcelProperty(value = "操作人", index = 9)
    private String applicant;


    public String getDetailTypeValue() {
        if (Objects.equals(type, 1)) {
            return InDetailTypeEnum.getInDetailType(detailType).getValue();
        }
        return OutDetailTypeEnum.getOutDetailType(detailType).getValue();
    }
}
