package com.sd.business.service.bom.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.bom.dto.BomClassifyDto;
import com.sd.business.entity.bom.dto.BomClassifySelectDto;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.vo.BomClassifyVo;
import com.sd.business.mapper.bom.BomClassifyMapper;
import com.sd.business.service.bom.BomClassifyService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import com.sd.framework.util.tree.TreeUtil;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;


/**
 * <p>
 * bom分类 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-04
 */
@Service
public class BomClassifyServiceImpl extends ServiceImpl<BomClassifyMapper, BomClassify> implements BomClassifyService {

    @Override
    public List<BomClassifyVo> tree(BomClassifySelectDto dto) {

        List<BomClassifyVo> list = Sql.create(BomClassifyVo.class)
                .selectAll(BomClassify.class)
                .from(BomClassify.class)
                .like(BomClassify::getName, dto.getName())
                .orderByDesc(BomClassify::getId)
                .list();

        return TreeUtil.build(list);
    }

    @Override
    public BomClassifyVo detail(Long id) {

        BomClassifyVo vo = Sql.create(BomClassifyVo.class)
                .selectAll(BomClassify.class)
                .from(BomClassify.class)
                .eq(BomClassify::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(BomClassifyDto dto) {
        long count = count(q -> q.eq(BomClassify::getCode, dto.getCode()).last("limit 1"));
        Assert.eqZero(count, "分类编码已存在");

        save(dto);
    }

    @Override
    public void edit(BomClassifyDto dto) {
        long count = count(q -> q.eq(BomClassify::getCode, dto.getCode()).ne(BaseIdPo::getId, dto.getId()).last("limit 1"));
        Assert.eqZero(count, "分类编码已存在");

        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

    @Override
    public List<Long> getChildrenIdList(Long parentId) {
        List<BomClassify> list = list();
        List<Long> childrenIdList = new ArrayList<>();
        getChildrenIdList(parentId, childrenIdList, list);
        return childrenIdList;
    }

    @Override
    public List<Long> getChildrenIdList(List<Long> parentIdList) {
        List<BomClassify> list = list();
        List<Long> childrenIdList = new ArrayList<>();
        parentIdList.forEach(item -> getChildrenIdList(item, childrenIdList, list));
        return childrenIdList;
    }

    /**
     * 递归添加子集节点id
     */
    private void getChildrenIdList(Long parentId, List<Long> childrenIdList, List<BomClassify> list) {
        childrenIdList.add(parentId);
        for (BomClassify bomClassify : list) {
            if (Objects.equals(bomClassify.getParentId(), parentId)) {
                getChildrenIdList(bomClassify.getId(), childrenIdList, list);
            }
        }
    }

}
