package com.sd.business.service.in.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.dto.InOutStorageBomSelectDto;
import com.sd.business.entity.in.po.InOutStorage;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.in.vo.InOutStorageBomVo;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.warehouse.po.Warehouse;
import com.sd.business.mapper.in.InOutStorageBomMapper;
import com.sd.business.service.in.InOutStorageBomService;
import com.sd.framework.util.sql.Sql;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 出入库bom 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class InOutStorageBomServiceImpl extends ServiceImpl<InOutStorageBomMapper, InOutStorageBom> implements InOutStorageBomService {

    @Override
    public Page<InOutStorageBomVo> getPage(InOutStorageBomSelectDto dto) {

        Page<InOutStorageBomVo> page = Sql.create(InOutStorageBomVo.class)
                .selectAll(InOutStorageBom.class)
                .select(InOutStorage::getInOutStorageTime,
                        InOutStorage::getDepartmentId,
                        InOutStorage::getType,
                        InOutStorage::getDetailType,
                        InOutStorage::getCode,
                        InOutStorage::getApplicant,
                        InOutStorage::getRemark)
                .selectAs(BomSpec::getCode, InOutStorageBomVo::getBomSpecCode)
                .selectAs(BomSpec::getName, InOutStorageBomVo::getBomSpecName)
                .selectAs(Warehouse::getType, InOutStorageBomVo::getWarehouseType)
                .selectAs(Warehouse::getName, InOutStorageBomVo::getWarehouseName)
                .selectAs(Warehouse::getCode, InOutStorageBomVo::getWarehouseCode)
                .selectAs(Department::getName, InOutStorageBomVo::getDepartmentName)
                .selectAs(Purchase::getCode, InOutStorageBomVo::getPurchaseCode)

                .from(InOutStorageBom.class)
                .leftJoin(InOutStorage.class, InOutStorageBom::getInOutStorageId)
                .leftJoin(BomSpec.class, InOutStorageBom::getBomSpecId)
                .leftJoin(Warehouse.class, InOutStorage::getWarehouseId)
                .leftJoin(Department.class, InOutStorage::getDepartmentId)
                .leftJoin(Purchase.class, InOutStorage::getPurchaseId)

                .orderByDesc(InOutStorageBom::getId)
                .like(InOutStorage::getCode, dto.getCode())
                .eq(InOutStorage::getDepartmentId, dto.getDepartmentId())
                .eq(InOutStorage::getType, dto.getType())
                .eq(InOutStorage::getDetailType, dto.getDetailType())
                .ge(InOutStorage::getInOutStorageTime, dto.getBeginTime())
                .le(InOutStorage::getInOutStorageTime, dto.getEndTime())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getName, dto.getBomSpecName())
                .notIn(BomSpec::getCode, "40904003", "409001", "40101010") // 筛选吊牌、不干胶、彩纸出入库记录
                .like(Purchase::getCode, dto.getPurchaseCode())
                .eq(Warehouse::getId, dto.getWarehouseId())

                .page(dto);

        List<InOutStorageBomVo> records = page.getRecords();
        if (records.isEmpty()) {
            return page;
        }

        records.forEach(item -> {
            if (item.getDepartmentId() == 0L) {
                item.setDepartmentName("胜德体育");
            }
        });

        return page;
    }

}
