package com.sd.business.service.inventory;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.service.BaseService;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.inventory.bo.InOutFun;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByDepartmentDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.dto.QuantityDto;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.entity.order.po.OrderSku;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * 库存 服务类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
public interface InventoryService extends BaseService<Inventory> {

    /**
     * 库存分页
     */
    Page<InventoryVo> getPage(InventorySelectDto dto);

    /**
     * 库存导出
     */
    void exportExcel(InventorySelectDto dto);

    /**
     * 查询库存
     */
    Map<Long, BigDecimal> getQuantity(QuantityDto dto);

    /**
     * 根据仓库获取库存数量
     */
    List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto);

    /**
     * 根据事业部获取库存数量
     */
    List<QuantityByDepartmentVo> getQuantityByDepartment(QuantityByDepartmentDto dto);

    /**
     * 锁定库存
     */
    boolean lockStorage(Map<Long, BigDecimal> map, Long warehouseId);

    /**
     * 锁定库存
     */
    boolean lockStorage(List<OrderSku> orderSkuList);

    /**
     * 解锁库存
     */
    void unlockStorage(Map<Long, BigDecimal> map, Long warehouseId);

    /**
     * 解锁库存
     */
    void unlockStorage(List<OrderSku> orderSkuList);

    /**
     * 采购入库
     */
    void purchaseInStorage(Long purchaseId, Long departmentId, Long warehouseId, List<InOutStorageBom> inOutStorageBomList);

    /**
     * 入库逻辑
     *
     * @param departmentId 事业部id
     * @param warehouseId  仓库id
     * @param list         入库明细
     */
    void in(Long departmentId, Long warehouseId, List<? extends InOutFun> list);

    /**
     * 出库逻辑
     *
     * @param departmentId 事业部id
     * @param warehouseId  仓库id
     * @param list         出库明细
     * @param lockStorage  是否从锁定库存出库
     */
    void out(Long departmentId, Long warehouseId, List<? extends InOutFun> list, Integer lockStorage);

}
