package com.sd.business.service.inventory.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.inventory.dto.InventoryFinishedOrderDto;
import com.sd.business.entity.inventory.dto.InventoryFinishedOrderSelectDto;
import com.sd.business.entity.inventory.enums.FinishedOperationTypeEnum;
import com.sd.business.entity.inventory.po.InventoryFinishedOrder;
import com.sd.business.entity.inventory.po.InventoryFinishedOrderDetail;
import com.sd.business.entity.inventory.vo.InventoryFinishedOrderVo;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.sku.po.SkuSpec;
import com.sd.business.mapper.inventory.InventoryFinishedOrderMapper;
import com.sd.business.service.inventory.InventoryFinishedOrderDetailService;
import com.sd.business.service.inventory.InventoryFinishedOrderService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 成品仓库订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class InventoryFinishedOrderServiceImpl extends ServiceImpl<InventoryFinishedOrderMapper, InventoryFinishedOrder> implements InventoryFinishedOrderService {

    @Autowired
    private InventoryFinishedOrderDetailService inventoryFinishedOrderDetailService;

    @Override
    public Page<InventoryFinishedOrderVo> getPage(InventoryFinishedOrderSelectDto dto) {

        Page<InventoryFinishedOrderVo> page = Sql.create(InventoryFinishedOrderVo.class)
                .select(InventoryFinishedOrderDetail::getQuantity,
                        InventoryFinishedOrderDetail::getCreateTime,
                        InventoryFinishedOrderDetail::getOperationType,
                        InventoryFinishedOrderDetail::getCode)
                .selectAs(SkuSpec::getCode, InventoryFinishedOrderVo::getSkuSpecCode)
                .selectAs(SkuSpec::getName, InventoryFinishedOrderVo::getSkuSpecName)
                .selectAs(OrderInfo::getCode, InventoryFinishedOrderVo::getOrderCode)
                .selectAs(OrderInfo::getWlnCode, InventoryFinishedOrderVo::getOrderWlnCode)
                .selectAs(Department::getName, InventoryFinishedOrderVo::getDepartmentName)

                .from(InventoryFinishedOrderDetail.class)
                .leftJoin(SkuSpec.class, InventoryFinishedOrderDetail::getSkuSpecId)
                .leftJoin(OrderInfo.class, InventoryFinishedOrderDetail::getOrderInfoId)
                .leftJoin(Department.class, OrderInfo::getDepartmentId)

                .orderByDesc(InventoryFinishedOrderDetail::getId)
                .like(InventoryFinishedOrderDetail::getCode, dto.getCode())
                .eq(SkuSpec::getId, dto.getSkuSpecId())
                .like(SkuSpec::getCode, dto.getSkuSpecCode())
                .like(SkuSpec::getName, dto.getSkuSpecName())
                .eq(OrderInfo::getId, dto.getOrderId())
                .like(OrderInfo::getCode, dto.getOrderCode())
                .like(OrderInfo::getWlnCode, dto.getOrderWlnCode())
                .eq(Department::getId, dto.getDepartmentId())
                .page(dto);

        return page;
    }

    @Override
    public InventoryFinishedOrderVo detail(Long id) {

        InventoryFinishedOrderVo vo = Sql.create(InventoryFinishedOrderVo.class)
                .selectAll(InventoryFinishedOrder.class)
                .from(InventoryFinishedOrder.class)
                .eq(InventoryFinishedOrder::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(InventoryFinishedOrderDto dto) {
        save(dto);
    }

    @Override
    public void edit(InventoryFinishedOrderDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

    @Override
    public void productionWarehousing(List<OrderSku> orderSkuList) {
        // 保存成品订单信息
        List<InventoryFinishedOrder> list = orderSkuList.stream().map(item -> {
            InventoryFinishedOrder inventoryFinishedOrder = new InventoryFinishedOrder();
            inventoryFinishedOrder.setDepartmentId(DepartmentConstant.SD_SPORTS);
            inventoryFinishedOrder.setOrderInfoId(item.getOrderId());
            inventoryFinishedOrder.setOrderSkuId(item.getId());
            inventoryFinishedOrder.setSkuSpecId(item.getSkuSpecId());
            inventoryFinishedOrder.setQuantity(item.getQuantity());
            inventoryFinishedOrder.setStatus(StatusConstant.YES);
            inventoryFinishedOrder.setExistingQuantity(item.getQuantity());
            return inventoryFinishedOrder;
        }).collect(Collectors.toList());
        saveBatch(list);

        // 添加流水记录
        inventoryFinishedOrderDetailService.add(list, FinishedOperationTypeEnum.PRODUCTION_WAREHOUSING);
    }

    @Override
    public void removeByOrderId(Long orderId) {
        remove(q -> q.eq(InventoryFinishedOrder::getOrderInfoId, orderId));
        inventoryFinishedOrderDetailService.remove(q -> q.eq(InventoryFinishedOrderDetail::getOrderInfoId, orderId));
    }

}
