package com.sd.business.service.inventory.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.exception.ServiceException;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.department.constant.DepartmentConstant;
import com.sd.business.entity.department.po.Department;
import com.sd.business.entity.in.po.InOutStorageBalance;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.inventory.bo.InOutFun;
import com.sd.business.entity.inventory.dto.InventorySelectDto;
import com.sd.business.entity.inventory.dto.QuantityByDepartmentDto;
import com.sd.business.entity.inventory.dto.QuantityByWarehouseDto;
import com.sd.business.entity.inventory.dto.QuantityDto;
import com.sd.business.entity.inventory.po.Inventory;
import com.sd.business.entity.inventory.vo.InventoryVo;
import com.sd.business.entity.inventory.vo.QuantityByDepartmentVo;
import com.sd.business.entity.inventory.vo.QuantityByWarehouseVo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.entity.warehouse.po.Warehouse;
import com.sd.business.mapper.inventory.InventoryMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageBalanceService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.framework.util.excel.util.ExcelUtil;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 库存 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class InventoryServiceImpl extends ServiceImpl<InventoryMapper, Inventory> implements InventoryService {

    @Autowired
    private HttpServletResponse response;

    @Autowired
    private PurchaseBomService purchaseBomService;

    @Autowired
    private InOutStorageBalanceService inOutStorageBalanceService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public Page<InventoryVo> getPage(InventorySelectDto dto) {

        return Sql.create(InventoryVo.class)
                .selectAll(Inventory.class)
                .selectAs(Warehouse::getName, InventoryVo::getWarehouseName)
                .selectAs(BomSpec::getCode, InventoryVo::getBomSpecCode)
                .selectAs(BomSpec::getName, InventoryVo::getBomSpecName)
                .selectAs(Department::getName, InventoryVo::getDepartmentName)
                .multiply(Inventory::getQuantity, Inventory::getBalanceUnitPrice, InventoryVo::getTotalPrice)
                .add(Inventory::getQuantity, Inventory::getLockQuantity, InventoryVo::getTotalQuantity)

                .from(Inventory.class)
                .leftJoin(BomSpec.class, Inventory::getBomSpecId)
                .leftJoin(Bom.class, BomSpec::getBomId)
                .leftJoin(Warehouse.class, Inventory::getWarehouseId)
                .leftJoin(Department.class, Inventory::getDepartmentId)

                .eq(Inventory::getDepartmentId, dto.getDepartmentId())
                .eq(Inventory::getWarehouseId, dto.getWarehouseId())
                .in(Inventory::getWarehouseId, dto.getWarehouseIdList())
                .like(BomSpec::getName, dto.getBomSpecName())
                .like(BomSpec::getCode, dto.getBomSpecCode())
                .like(BomSpec::getColour, dto.getBomSpecColour())
                .like(BomSpec::getLength, dto.getBomSpecLength())
                .like(BomSpec::getWidth, dto.getBomSpecWidth())
                .eq(Bom::getBomClassifyId, dto.getBomClassifyId())
                .eq(Bom::getSpecies, dto.getBomSpecies())
                .eq(Bom::getChromatophore, dto.getBomChromatophore())
                .eq(Bom::getEmbossingProcess, dto.getBomEmbossingProcess())
                .eq(Bom::getFrontGrain, dto.getBomFrontGrain())
                .eq(Bom::getReverseGrain, dto.getBomReverseGrain())
                .orderByDesc(Inventory::getId)
                .page(dto);
    }

    @Override
    public void exportExcel(InventorySelectDto dto) {
        dto.setSearchAll(true);
        Page<InventoryVo> page = getPage(dto);
        List<InventoryVo> list = page.getRecords();
        ExcelUtil.export(response, DateUtil.format(new Date(), "yyyy-MM-dd库存"), "库存", list, InventoryVo.class);
    }

    @Override
    public Map<Long, BigDecimal> getQuantity(QuantityDto dto) {
        return mapKV(Inventory::getBomSpecId, Inventory::getQuantity, q -> q
                .eq(Inventory::getWarehouseId, dto.getWarehouseId())
                .eq(Inventory::getDepartmentId, dto.getDepartmentId())
                .in(Inventory::getBomSpecId, dto.getBomSpecIdList()));
    }

    @Override
    public List<QuantityByWarehouseVo> getQuantityByWarehouse(QuantityByWarehouseDto dto) {

        return Sql.create(QuantityByWarehouseVo.class)
                .sum(Inventory::getQuantity, QuantityByWarehouseVo::getInventoryQuantity)
                .selectAs(Warehouse::getId, QuantityByWarehouseVo::getWarehouseId)
                .selectAs(Warehouse::getName, QuantityByWarehouseVo::getWarehouseName)

                .from(Inventory.class)
                .innerJoin(Warehouse.class, Warehouse::getId, Inventory::getId)

                .eq(Inventory::getDepartmentId, dto.getDepartmentId())
                .groupBy(Warehouse::getId)
                .orderByAsc(Warehouse::getId)
                .list();
    }

    @Override
    public List<QuantityByDepartmentVo> getQuantityByDepartment(QuantityByDepartmentDto dto) {

        return Sql.create(QuantityByDepartmentVo.class)
                .sum(Inventory::getQuantity, QuantityByDepartmentVo::getInventoryQuantity)
                .selectAs(Department::getId, QuantityByDepartmentVo::getDepartmentId)
                .selectAs(Department::getName, QuantityByDepartmentVo::getDepartmentName)

                .from(Inventory.class)
                .innerJoin(Department.class, Department::getId, Inventory::getDepartmentId)

                .eq(Inventory::getWarehouseId, dto.getWarehouseId())
                .groupBy(Department::getId)
                .orderByAsc(Department::getId)
                .list();
    }

    @Override
    public boolean lockStorage(Map<Long, BigDecimal> map, Long warehouseId) {
        if (ObjectUtil.isEmpty(map)) {
            return true;
        }
        synchronized (this) {
            List<Inventory> list = list(q -> q
                    .in(Inventory::getBomSpecId, map.keySet())
                    .eq(Inventory::getWarehouseId, warehouseId)
                    .eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
            );
            if (map.size() != list.size()) {
                return false;
            }
            for (Inventory inventory : list) {
                BigDecimal lockQuantity = map.get(inventory.getBomSpecId());
                if (inventory.getQuantity().compareTo(lockQuantity) >= 0) {
                    inventory.setQuantity(inventory.getQuantity().subtract(lockQuantity));
                    inventory.setLockQuantity(inventory.getLockQuantity().add(lockQuantity));
                } else {
                    return false;
                }
            }
            this.updateBatchById(list);
        }
        return true;
    }

    @Override
    public boolean lockStorage(List<OrderSku> orderSkuList) {
        Map<Long, BigDecimal> map = orderSkuList.stream()
                .collect(Collectors.toMap(OrderSku::getBomSpecId, OrderSku::getQuantity, BigDecimal::add));
        return lockStorage(map, WarehouseConstant.SEMI_FINISHED_PRODUCT);
    }

    @Override
    public void unlockStorage(Map<Long, BigDecimal> map, Long warehouseId) {
        if (ObjectUtil.isEmpty(map)) {
            return;
        }
        synchronized (this) {
            List<Inventory> list = list(q -> q
                    .in(Inventory::getBomSpecId, map.keySet())
                    .eq(Inventory::getWarehouseId, warehouseId)
                    .eq(Inventory::getDepartmentId, DepartmentConstant.SD_SPORTS)
            );
            for (Inventory inventory : list) {
                BigDecimal lockQuantity = map.get(inventory.getBomSpecId());
                inventory.setQuantity(inventory.getQuantity().add(lockQuantity));
                inventory.setLockQuantity(inventory.getLockQuantity().subtract(lockQuantity));
            }
            updateBatchById(list);
        }
    }

    @Override
    public void unlockStorage(List<OrderSku> orderSkuList) {
        if (orderSkuList.isEmpty()) {
            return;
        }
        Map<Long, BigDecimal> map = orderSkuList.stream()
                .collect(Collectors.toMap(OrderSku::getBomSpecId, OrderSku::getQuantity, BigDecimal::add));
        unlockStorage(map, WarehouseConstant.SEMI_FINISHED_PRODUCT);
    }

    @Override
    public void purchaseInStorage(Long purchaseId, Long departmentId, Long warehouseId, List<InOutStorageBom> inOutStorageBomList) {
        List<Long> bomSpecIdList = inOutStorageBomList.stream().map(InOutStorageBom::getBomSpecId).collect(Collectors.toList());
        synchronized (this) {
            // 通过事业部id和出库id获取bom规格库存
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);

            // 通过bom规格分组
            Map<Long, Inventory> map = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            Map<Long, PurchaseBom> purchaseBomMap = purchaseBomService.mapKEntity(PurchaseBom::getBomSpecId,
                    q -> q.eq(PurchaseBom::getPurchaseId, purchaseId).in(PurchaseBom::getBomSpecId, bomSpecIdList));

            // 保存结存单价批次
            List<InOutStorageBalance> inOutStorageDetailsList = inOutStorageBomList.stream().map(item -> {
                // 获取入库单价
                PurchaseBom purchaseBom = purchaseBomMap.get(item.getBomSpecId());
                BigDecimal inUnitPrice = purchaseBom == null ? BigDecimal.ZERO : purchaseBom.getUnitPrice();
                item.setUnitPrice(inUnitPrice);

                InOutStorageBalance inOutStorageBalance = new InOutStorageBalance();
                inOutStorageBalance.setDepartmentId(departmentId);
                inOutStorageBalance.setWarehouseId(warehouseId);
                inOutStorageBalance.setBomSpecId(item.getBomSpecId());
                inOutStorageBalance.setInUnitPrice(inUnitPrice);
                inOutStorageBalance.setQuantity(item.getQuantity());
                return inOutStorageBalance;
            }).collect(Collectors.toList());
            inOutStorageBalanceService.saveBatch(inOutStorageDetailsList);

            for (InOutStorageBom inOutStorageBom : inOutStorageBomList) {

                // 库存
                Inventory inventory = map.computeIfAbsent(inOutStorageBom.getBomSpecId(), item -> {
                    Inventory tempInventory = new Inventory();
                    tempInventory.setWarehouseId(warehouseId);
                    tempInventory.setDepartmentId(departmentId);
                    tempInventory.setBomSpecId(item);
                    tempInventory.setQuantity(BigDecimal.ZERO);
                    tempInventory.setLockQuantity(BigDecimal.ZERO);
                    tempInventory.setBalanceUnitPrice(BigDecimal.ZERO);
                    inventoryList.add(tempInventory);
                    return tempInventory;
                });

                PurchaseBom purchaseBom = purchaseBomMap.get(inOutStorageBom.getBomSpecId());
                // 实际库存数  当前库存数量加锁定库存
                BigDecimal actualInventoryQuantity = inventory.getQuantity().add(inventory.getLockQuantity());
                // 结存数量
                BigDecimal sumQuantity = actualInventoryQuantity.add(inOutStorageBom.getQuantity());
                // 结存金额
                BigDecimal sumPrice = actualInventoryQuantity.multiply(inventory.getBalanceUnitPrice())
                        .add(inOutStorageBom.getQuantity().multiply(purchaseBom.getUnitPrice()));

                // 获取入库结存单价
                BigDecimal balanceUnitPrice = sumPrice.divide(sumQuantity, 3, RoundingMode.HALF_UP);

                inventory.setQuantity(sumQuantity.subtract(inventory.getLockQuantity()));
                inventory.setBalanceUnitPrice(balanceUnitPrice);

                inOutStorageBom.setBalanceInventoryQuantity(sumQuantity);
            }

            saveOrUpdateBatch(inventoryList);
        }
    }

    @Override
    public void in(Long departmentId, Long warehouseId, List<? extends InOutFun> list) {
        if (list.isEmpty()) {
            return;
        }

        List<Long> bomSpecIdList = list.stream().map(InOutFun::getBomSpecId).collect(Collectors.toList());

        synchronized (this) {

            // 通过事业部id和出库id获取bom规格库存
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);

            // 通过bom规格分组
            Map<Long, Inventory> map = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            // 保存结存单价批次
            List<InOutStorageBalance> inOutStorageBalanceList = list.stream().map(item -> {

                // todo 获取临时结存单价
                Inventory inventory = map.get(item.getBomSpecId());
                BigDecimal inUnitPrice = inventory == null ? BigDecimal.ZERO : inventory.getBalanceUnitPrice();

                InOutStorageBalance inOutStorageBalance = new InOutStorageBalance();
                inOutStorageBalance.setDepartmentId(departmentId);
                inOutStorageBalance.setWarehouseId(warehouseId);
                inOutStorageBalance.setBomSpecId(item.getBomSpecId());
                inOutStorageBalance.setInUnitPrice(inUnitPrice);
                inOutStorageBalance.setQuantity(item.getQuantity());

                return inOutStorageBalance;
            }).collect(Collectors.toList());
            inOutStorageBalanceService.saveBatch(inOutStorageBalanceList);

            // 通过出库id获取bom规格结存列表
            Map<Long, List<InOutStorageBalance>> inOutStorageBalanceMap = getInOutStorageBalanceMap(warehouseId, bomSpecIdList);

            for (InOutFun inOutFun : list) {

                // 库存
                Inventory inventory = map.computeIfAbsent(inOutFun.getBomSpecId(), item -> {
                    Inventory tempInventory = new Inventory();
                    tempInventory.setWarehouseId(warehouseId);
                    tempInventory.setDepartmentId(departmentId);
                    tempInventory.setBomSpecId(item);
                    tempInventory.setQuantity(BigDecimal.ZERO);
                    tempInventory.setLockQuantity(BigDecimal.ZERO);
                    inventoryList.add(tempInventory);
                    return tempInventory;
                });

                // 获取入库结存单价
                BigDecimal balanceUnitPrice = getBalanceUnitPrice(inOutStorageBalanceMap, inOutFun);

                inventory.setQuantity(inventory.getQuantity().add(inOutFun.getQuantity()));
                inventory.setBalanceUnitPrice(balanceUnitPrice);

                // 保存当前入库记录的结存库存
                inOutFun.setBalanceInventoryQuantity(inventory.getQuantity().add(inventory.getLockQuantity()));
            }

            saveOrUpdateBatch(inventoryList);
        }
    }

    @Override
    public void out(Long departmentId, Long warehouseId, List<? extends InOutFun> list, Integer lockStorage) {
        List<Long> bomSpecIdList = list.stream().map(InOutFun::getBomSpecId).collect(Collectors.toList());

        synchronized (this) {

            // 通过事业部id和出库id获取bom规格库存
            List<Inventory> inventoryList = getInventoryList(departmentId, warehouseId, bomSpecIdList);
            Map<Long, Inventory> inventoryMap = inventoryList.stream().collect(Collectors.toMap(Inventory::getBomSpecId, Function.identity()));

            // 获取bom规格id和结存明细列表map
            Map<Long, List<InOutStorageBalance>> inOutStorageBalanceMap = getInOutStorageBalanceMap(warehouseId, bomSpecIdList);

            for (InOutFun inOutFun : list) {

                BigDecimal quantity = inOutFun.getQuantity();
                Long bomSpecId = inOutFun.getBomSpecId();
                Inventory inventory = inventoryMap.get(bomSpecId);

                // 出库冻结库存
                if (Objects.equals(lockStorage, StatusConstant.YES)) {
                    inventory.setLockQuantity(inventory.getLockQuantity().subtract(quantity));
                }
                // 出库非冻结库存
                else {
                    if (inventory == null || inventory.getQuantity().compareTo(quantity) < 0) {
                        BomSpec bomSpec = bomSpecService.getById(bomSpecId);
                        if (bomSpec == null) {
                            throw new ServiceException("未知bom规格id：" + bomSpecId);
                        }
                        throw new ServiceException("出库失败,品号为：" + bomSpec.getCode() + " 的bom库存不足,库存为：" +
                                (inventory == null ? BigDecimal.ZERO : inventory.getQuantity()) + ",需要出库：" + quantity);
                    }
                    inventory.setQuantity(inventory.getQuantity().subtract(quantity));
                }

                // 计算出库结存单价
                BigDecimal balanceUnitPrice = getOutBalanceUnitPrice(inOutStorageBalanceMap, inOutFun);

                // 赋值结存单价
                inventory.setBalanceUnitPrice(balanceUnitPrice);

                // 保存当前入库记录的结存库存
                inOutFun.setBalanceInventoryQuantity(inventory.getQuantity().add(inventory.getLockQuantity()));

            }

            // 修改计算结存单价的出入库明细表
            List<InOutStorageBalance> inOutStorageDetailsList = inOutStorageBalanceMap.values()
                    .stream().flatMap(Collection::stream).collect(Collectors.toList());
            inOutStorageBalanceService.updateBatchById(inOutStorageDetailsList);

            // 修改库存数量以及结存单价
            updateBatchById(inventoryList);
        }
    }

    /**
     * 通过出库id获取bom规格结存列表
     */
    private Map<Long, List<InOutStorageBalance>> getInOutStorageBalanceMap(Long warehouseId, List<Long> bomSpecIdList) {
        return inOutStorageBalanceService.mapKGroup(InOutStorageBalance::getBomSpecId, q -> q
                .eq(InOutStorageBalance::getWarehouseId, warehouseId)
                .in(InOutStorageBalance::getBomSpecId, bomSpecIdList)
                .gt(InOutStorageBalance::getQuantity, BigDecimal.ZERO)
        );
    }

    /**
     * 通过事业部id和出库id获取bom规格库存
     */
    private List<Inventory> getInventoryList(Long departmentId, Long warehouseId, List<Long> bomSpecIdList) {
        return list(q -> q
                .eq(Inventory::getDepartmentId, departmentId)
                .eq(Inventory::getWarehouseId, warehouseId)
                .in(Inventory::getBomSpecId, bomSpecIdList)
        );
    }

    /**
     * 获取结存单价
     */
    private BigDecimal getBalanceUnitPrice(Map<Long, List<InOutStorageBalance>> map, InOutFun inOutFun) {

        List<InOutStorageBalance> inOutStorageBalanceList = map.get(inOutFun.getBomSpecId());

        // 没有结存记录，结存单价为0
        if (ObjectUtil.isEmpty(inOutStorageBalanceList)) {
            return BigDecimal.ZERO;
        }

        // 获取结存数量
        BigDecimal sumQuantity = inOutStorageBalanceList.stream()
                .map(InOutStorageBalance::getQuantity)
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 数量为0，结存单价为0
        if (sumQuantity.compareTo(BigDecimal.ZERO) == 0) {
            return BigDecimal.ZERO;
        }

        // 获取结存金额
        BigDecimal sumPrice = inOutStorageBalanceList.stream()
                .map(item -> item.getQuantity().multiply(item.getInUnitPrice()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        // 计算新结存单价
        return sumPrice.divide(sumQuantity, 3, RoundingMode.HALF_UP);
    }

    /**
     * 获取出库结存单价
     */
    private BigDecimal getOutBalanceUnitPrice(Map<Long, List<InOutStorageBalance>> map, InOutFun inOutFun) {

        // 出库数量
        BigDecimal outQuantity = inOutFun.getQuantity();

        // 结存列表
        List<InOutStorageBalance> inOutStorageBalanceList = map.get(inOutFun.getBomSpecId());

        // 没有结存记录，结存单价为0
        if (ObjectUtil.isEmpty(inOutStorageBalanceList)) {
            return BigDecimal.ZERO;
        }

        for (InOutStorageBalance inOutStorageBalance : inOutStorageBalanceList) {
            // 可出库数量
            BigDecimal quantity = inOutStorageBalance.getQuantity();

            if (outQuantity.compareTo(quantity) > 0) {
                inOutStorageBalance.setQuantity(BigDecimal.ZERO);
                outQuantity = outQuantity.subtract(quantity);
            } else {
                inOutStorageBalance.setQuantity(quantity.subtract(outQuantity));
                break;
            }
        }

        return getBalanceUnitPrice(map, inOutFun);
    }


}
