package com.sd.business.service.order.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.sd.business.entity.bom.bo.BomSpecBo;
import com.sd.business.entity.in.dto.InOutStorageDto;
import com.sd.business.entity.in.emums.InDetailTypeEnum;
import com.sd.business.entity.in.emums.InOutTypeEnum;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.order.dto.OrderInfoDto;
import com.sd.business.entity.order.dto.OrderInfoSelectDto;
import com.sd.business.entity.order.enums.OrderStatusEnum;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderPackageBom;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.order.po.OrderSkuProductionCost;
import com.sd.business.entity.order.vo.OrderInfoVo;
import com.sd.business.entity.production.po.ProductionWorkOrder;
import com.sd.business.entity.warehouse.constant.WarehouseConstant;
import com.sd.business.mapper.order.OrderInfoMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.in.InOutStorageService;
import com.sd.business.service.inventory.InventoryFinishedService;
import com.sd.business.service.inventory.InventoryService;
import com.sd.business.service.order.OrderInfoService;
import com.sd.business.service.order.OrderPackageBomService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuProductionCostService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.production.ProductionWorkOrderService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 订单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class OrderInfoServiceImpl extends ServiceImpl<OrderInfoMapper, OrderInfo> implements OrderInfoService {

    @Autowired
    private BomSpecService bomSpecService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private OrderPackageBomService orderPackageBomService;

    @Autowired
    private OrderSkuProductionCostService orderSkuProductionCostService;

    @Autowired
    private ProductionWorkOrderService productionWorkOrderService;

    @Autowired
    private InventoryService inventoryService;

    @Autowired
    private InOutStorageService inOutStorageService;

    @Autowired
    private InventoryFinishedService inventoryFinishedService;

    @Override
    public Page<OrderInfoVo> getPage(OrderInfoSelectDto dto) {

        Page<OrderInfoVo> page = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .from(OrderInfo.class)
                .orderByDesc(OrderInfo::getId)
                .page(dto);

        return page;
    }

    @Override
    public OrderInfoVo detail(Long id) {

        OrderInfoVo vo = Sql.create(OrderInfoVo.class)
                .selectAll(OrderInfo.class)
                .from(OrderInfo.class)
                .eq(OrderInfo::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(OrderInfoDto dto) {
        save(dto);
    }

    @Override
    public void edit(OrderInfoDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
        orderSkuService.remove(q -> q.eq(OrderSku::getOrderId, id));
        orderSkuBomService.remove(q -> q.eq(OrderSkuBom::getOrderId, id));
        orderPackageBomService.remove(q -> q.eq(OrderPackageBom::getOrderId, id));
        orderSkuProductionCostService.remove(q -> q.eq(OrderSkuProductionCost::getOrderId, id));
        productionWorkOrderService.remove(q -> q.eq(ProductionWorkOrder::getOrderId, id));

        ObsFileUtil.removeFile(id);
    }

    @DSTransactional
    @Override
    public synchronized void deleteAndStore(Long id) {
        OrderInfo orderInfo = getById(id);

        if (orderInfo == null) {
            throw new ServiceException("未找到订单");
        }

        // 解锁库存
        unLockStorage(orderInfo);

        if (orderInfo.getStatus() <= OrderStatusEnum.STOCK_PREPARATION.getKey()) {
            delete(id);
            return;
        }

        // 获取订单商品主材和包材
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, id));
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.eq(OrderSkuBom::getOrderId, id));

        Map<Long, OrderSku> orderSkuMap = orderSkuList.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));

        // 合并主材和包材
        List<InOutStorageBom> inOutStorageBomList = orderSkuList.stream().map(item -> {
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(item.getBomSpecId());
            inOutStorageBom.setQuantity(item.getQuantity());
            return inOutStorageBom;
        }).collect(Collectors.toList());

        for (OrderSkuBom orderSkuBom : orderSkuBomList) {
            OrderSku orderSku = orderSkuMap.get(orderSkuBom.getOrderSkuId());
            InOutStorageBom inOutStorageBom = new InOutStorageBom();
            inOutStorageBom.setBomSpecId(orderSkuBom.getBomSpecId());
            inOutStorageBom.setQuantity(orderSkuBom.getQuantity().multiply(orderSku.getQuantity()));
            inOutStorageBomList.add(inOutStorageBom);
        }

        // 合并数量
        Map<Long, InOutStorageBom> map = inOutStorageBomList.stream().collect(Collectors.toMap(
                InOutStorageBom::getBomSpecId,
                Function.identity(),
                (v1, v2) -> {
                    v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                    return v1;
                }
        ));
        inOutStorageBomList = new ArrayList<>(map.values());

        // 区分半成品仓和包材仓库
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(map.keySet());

        // 主材从半成品仓入库
        semiFinishedProductOutStorage(inOutStorageBomList, bomSpecBoMap, orderInfo);

        // 包材从包材仓出库
        packagingMaterialOutStorage(inOutStorageBomList, bomSpecBoMap, orderInfo);

        // 删除订单回滚成品仓库存
        inventoryFinishedService.removeOrder(id);

        delete(id);
    }

    /**
     * 解锁库存
     */
    private void unLockStorage(OrderInfo orderInfo) {

        // 未锁定库存跳过
        if (ObjectUtil.notEqual(orderInfo.getLockStorage(), StatusConstant.YES)) {
            return;
        }

        // 修改订单库存锁定状态为否
        orderInfo.setLockStorage(StatusConstant.NO);
        update(q -> q.eq(BaseIdPo::getId, orderInfo.getId()).set(OrderInfo::getLockStorage, StatusConstant.NO));

        // 订单sku列表
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.eq(OrderSku::getOrderId, orderInfo.getId()));

        // 过滤订单
        List<Long> bomSpecIdList = orderSkuList.stream().map(OrderSku::getBomSpecId).collect(Collectors.toList());
        Map<Long, BomSpecBo> bomSpecBoMap = bomSpecService.getBomSpecBo(bomSpecIdList);
        orderSkuList = orderSkuList.stream()
                .filter(item -> bomSpecBoMap.get(item.getBomSpecId()).getClassifyParentId().equals(1L))
                .collect(Collectors.toList());

        inventoryService.unlockStorage(orderSkuList);
    }

    /**
     * 主材出库
     */
    private void semiFinishedProductOutStorage(List<InOutStorageBom> inOutStorageBomList, Map<Long, BomSpecBo> bomSpecBoMap, OrderInfo orderInfo) {
        List<InOutStorageBom> semiFinishedProductInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return false;
                    }
                    return ObjectUtil.equal(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto semiFinishedProductInOutStorageDto = new InOutStorageDto();
        semiFinishedProductInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        semiFinishedProductInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        semiFinishedProductInOutStorageDto.setWarehouseId(WarehouseConstant.SEMI_FINISHED_PRODUCT);
        semiFinishedProductInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        semiFinishedProductInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        semiFinishedProductInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 主材退料入库");
        semiFinishedProductInOutStorageDto.setInOutStorageBomList(semiFinishedProductInOutStorageBomList);
        inOutStorageService.add(semiFinishedProductInOutStorageDto);
    }

    /**
     * 包材出库
     */
    private void packagingMaterialOutStorage(List<InOutStorageBom> inOutStorageBomList, Map<Long, BomSpecBo> bomSpecBoMap, OrderInfo orderInfo) {
        List<InOutStorageBom> packagingMaterialInOutStorageBomList = inOutStorageBomList
                .stream()
                .filter(item -> {
                    BomSpecBo tempBomSpecBo = bomSpecBoMap.get(item.getBomSpecId());
                    if (tempBomSpecBo == null) {
                        return true;
                    }
                    return ObjectUtil.notEqual(tempBomSpecBo.getClassifyParentId(), 1L);
                })
                .collect(Collectors.toList());
        InOutStorageDto packagingMaterialInOutStorageDto = new InOutStorageDto();
        packagingMaterialInOutStorageDto.setType(InOutTypeEnum.IN.getKey());
        packagingMaterialInOutStorageDto.setDetailType(InDetailTypeEnum.RETURN_GOODS.getKey());
        packagingMaterialInOutStorageDto.setWarehouseId(WarehouseConstant.PACKAGING_MATERIAL);
        packagingMaterialInOutStorageDto.setDepartmentId(orderInfo.getOutDepartmentId());
        packagingMaterialInOutStorageDto.setApplicant(SecurityUtils.getLoginUser().getUser().getNickName());
        packagingMaterialInOutStorageDto.setRemark("删除订单：" + orderInfo.getCode()
                + (StrUtil.isBlank(orderInfo.getWlnCode()) ? StringPool.EMPTY : "（" + orderInfo.getWlnCode() + "）")
                + " 包材退料入库");
        packagingMaterialInOutStorageDto.setInOutStorageBomList(packagingMaterialInOutStorageBomList);
        inOutStorageService.add(packagingMaterialInOutStorageDto);
    }

}
