package com.sd.business.service.purchase.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.sd.business.entity.in.po.InOutStorageBom;
import com.sd.business.entity.purchase.dto.PurchaseDto;
import com.sd.business.entity.purchase.dto.PurchaseSelectDto;
import com.sd.business.entity.purchase.enums.PurchaseStatusEnum;
import com.sd.business.entity.purchase.po.Purchase;
import com.sd.business.entity.purchase.po.PurchaseBom;
import com.sd.business.entity.purchase.vo.PurchaseVo;
import com.sd.business.mapper.purchase.PurchaseMapper;
import com.sd.business.service.purchase.PurchaseBomService;
import com.sd.business.service.purchase.PurchaseService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;


/**
 * <p>
 * 采购合同 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-12
 */
@Service
public class PurchaseServiceImpl extends ServiceImpl<PurchaseMapper, Purchase> implements PurchaseService {

    @Autowired
    private PurchaseBomService purchaseBomService;

    @Override
    public Page<PurchaseVo> getPage(PurchaseSelectDto dto) {

        Page<PurchaseVo> page = Sql.create(PurchaseVo.class)
                .selectAll(Purchase.class)
                .from(Purchase.class)
                .orderByDesc(Purchase::getId)
                .page(dto);

        return page;
    }

    @Override
    public PurchaseVo detail(Long id) {

        PurchaseVo vo = Sql.create(PurchaseVo.class)
                .selectAll(Purchase.class)
                .from(Purchase.class)
                .eq(Purchase::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(PurchaseDto dto) {
        save(dto);
    }

    @Override
    public void edit(PurchaseDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

    @Override
    public void updatePurchaseBomArrivalQuantity(Long purchaseId, List<InOutStorageBom> inOutStorageBomList) {

        for (InOutStorageBom inOutStorageBom : inOutStorageBomList) {

            PurchaseBom bom = purchaseBomService.getOne(q -> q
                    .eq(PurchaseBom::getPurchaseId, purchaseId)
                    .eq(PurchaseBom::getBomSpecId, inOutStorageBom.getBomSpecId()));

            if (ObjectUtil.isEmpty(bom)) {
                continue;
            }

            purchaseBomService.update(q -> q
                    .eq(PurchaseBom::getId, bom.getId())
                    .set(PurchaseBom::getArrivalQuantity, bom.getArrivalQuantity().add(inOutStorageBom.getQuantity())));

            // 查询当前采购在途数量
            long count = purchaseBomService.count(q -> q
                    .eq(PurchaseBom::getPurchaseId, purchaseId)
                    .apply("(purchase_quantity + return_quantity) > arrival_quantity")
                    .last("limit 1"));

            if (count == 0) {
                // 如果为0，将入库状态设置为已完成
                this.update(q -> q
                        .set(Purchase::getStorageStatus, StatusConstant.YES)
                        .set(Purchase::getStatus, PurchaseStatusEnum.HAVE_PURCHASE.getKey())
                        .eq(Purchase::getId, purchaseId));
            }
        }

    }

}
