package com.sd.business.service.statement.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.entity.statement.dto.StatementOfAccountSelectDto;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.statement.vo.StatementOfAccountVo;
import com.sd.business.mapper.statement.StatementOfAccountMapper;
import com.sd.business.service.order.OrderInfoService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;


/**
 * <p>
 * 对账单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-12-11
 */
@Service
public class StatementOfAccountServiceImpl extends ServiceImpl<StatementOfAccountMapper, StatementOfAccount> implements StatementOfAccountService {

    @Autowired
    private OrderInfoService orderInfoService;

    @Override
    public Page<StatementOfAccountVo> getPage(StatementOfAccountSelectDto dto) {

        Page<StatementOfAccountVo> page = Sql.create(StatementOfAccountVo.class)
                .selectAll(StatementOfAccount.class)
                .from(StatementOfAccount.class)
                .orderByDesc(StatementOfAccount::getId)
                .page(dto);

        return page;
    }

    @Override
    public StatementOfAccountVo detail(Long id) {

        StatementOfAccountVo vo = Sql.create(StatementOfAccountVo.class)
                .selectAll(StatementOfAccount.class)
                .from(StatementOfAccount.class)
                .eq(StatementOfAccount::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(StatementOfAccountDto dto) {
        List<Long> orderIdList = dto.getOrderIdList();

        // 判断当前事业部今日是否生成对账单
        Long departmentId = dto.getDepartmentId();
        Date date = new Date();
        Date beginDate = DateUtil.beginOfDay(date);
        Date endDate = DateUtil.endOfDay(date);
        StatementOfAccount statementOfAccount = this.getOne(q -> q
                .eq(StatementOfAccount::getDepartmentId, departmentId)
                .between(StatementOfAccount::getTimePeriod, beginDate, endDate));

        if (statementOfAccount == null) {
            statementOfAccount = new StatementOfAccount();
            statementOfAccount.setDepartmentId(departmentId);
            statementOfAccount.setCode(CodeEnum.STATEMENT_OF_ACCOUNT_CODE.getCode());
            statementOfAccount.setType(2);
            statementOfAccount.setTimePeriod(new Date());
            this.save(statementOfAccount);
        }
        Long accountId = statementOfAccount.getId();

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderInfoService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, accountId)
            );
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(StatementOfAccountDto dto) {
        Long statementOfAccountId = dto.getId();
        List<Long> orderIdList = dto.getOrderIdList();

        Assert.notNull(statementOfAccountId, "对账单id不能为空");

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderInfoService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, statementOfAccountId)
            );
        }

        orderInfoService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, statementOfAccountId)
                .notIn(ObjectUtil.isNotEmpty(orderIdList), BaseIdPo::getId, orderIdList)
                .set(OrderInfo::getStatementOfAccountId, null)
        );
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(Long id) {
        orderInfoService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, id)
                .set(OrderInfo::getStatementOfAccountId, null)
        );

        removeById(id);
        ObsFileUtil.removeFile(id);
    }

}
