package com.sd.framework.util;

import cn.hutool.extra.spring.SpringUtil;
import com.ruoyi.common.exception.ServiceException;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.function.Supplier;

@Slf4j
public class TransactionUtil {

    private final static TransactionTemplate transactionTemplate = SpringUtil.getBean(TransactionTemplate.class);

    public static void execute(Runnable runnable) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(@NotNull TransactionStatus status) {
                try {
                    runnable.run();
                } catch (Exception e) {
                    status.setRollbackOnly();
                    throw e;
                }
            }
        });
    }

    public static void execute(Runnable runnable, String errorStr) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(@NotNull TransactionStatus status) {
                try {
                    runnable.run();
                } catch (Exception e) {
                    status.setRollbackOnly();
                    log.error(errorStr, e);
                    throw new ServiceException(errorStr);
                }
            }
        });
    }

    public static <T> T execute(Supplier<T> supplier) {
        return transactionTemplate.execute(status -> {
            try {
                return supplier.get();
            } catch (Exception e) {
                status.setRollbackOnly();
                throw e;
            }
        });
    }

    public static <T> T execute(Supplier<T> supplier, String errorStr) {
        return transactionTemplate.execute(status -> {
            try {
                return supplier.get();
            } catch (Exception e) {
                status.setRollbackOnly();
                log.error(errorStr, e);
                throw new ServiceException(errorStr);
            }
        });
    }

}
